#include <proto.h>

#include "gtshell.h"
#include "gtcursor.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtShell class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init shell.
*
\************************************************************************/
GtShell *GtMkShell(char *n,GtManager *p,GtTag *tgs)
{
	GtShell *obj;
	
	if(!(obj=new GtShell()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtShell object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init shell.
*
\************************************************************************/
GtShell *GtMkShell(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtShell *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkShell(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty shell.
*
\************************************************************************/
GtShell::GtShell(void)
{
	sminxs=sminys=1;
	smaxxs=smaxys=GTS_MAX;
	maxchild=1;
}

/************************************************************************\
*
* Setup shell.
*
\************************************************************************/
sbool GtShell::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtBdrMan::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_MINXS,GTRT_WORD,&sminxs);
	app->GetRes(this,GTR_MINYS,GTRT_WORD,&sminys);
	app->GetRes(this,GTR_MAXXS,GTRT_WORD,&smaxxs);
	app->GetRes(this,GTR_MAXYS,GTRT_WORD,&smaxys);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_MinXSize:
				sminxs=t->d.val;
				break;
			case GT_MinYSize:
				sminys=t->d.val;
				break;
			case GT_MaxXSize:
				smaxxs=t->d.val;
				break;
			case GT_MaxYSize:
				smaxys=t->d.val;
				break;
		}
	}
	if(!gfx->cur)
	{
		if(!(gfx->cur=GtMkSharedCursor(dsp,
				GT_Shape,XC_top_left_arrow,
				GT_DONE)))
		{
			GtError(this,GTEC_NOXRESOURCE,"No GtCursor!\n");
			return(FALSE);
		}
		gfx->win->SetV(GT_Cursor,gfx->cur,GT_DONE);
	}
	sminxs=GTMAX(sminxs,1);
	sminys=GTMAX(sminys,1);
	smaxxs=GTMAX(sminxs,smaxxs);
	smaxys=GTMAX(sminys,smaxys);
	this->SetMinMaxSize(sminxs,sminys,smaxxs,smaxys);
	return(TRUE);
}

/************************************************************************\
*
* Set shell attributes.
*
\************************************************************************/
sbool GtShell::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE;
	
	if(!GtBdrMan::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_MinXSize:
				sminxs=t->d.val;
				ch=TRUE;
				break;
			case GT_MinYSize:
				sminys=t->d.val;
				ch=TRUE;
				break;
			case GT_MaxXSize:
				smaxxs=t->d.val;
				ch=TRUE;
				break;
			case GT_MaxYSize:
				smaxys=t->d.val;
				ch=TRUE;
				break;
		}
	}
	if(ch)
	{
		sminxs=GTMAX(sminxs,1);
		sminys=GTMAX(sminys,1);
		smaxxs=GTMAX(sminxs,smaxxs);
		smaxys=GTMAX(sminys,smaxys);
		this->Relayout();
	}
	return(TRUE);
}

/************************************************************************\
*
* Get shell attributes.
*
\************************************************************************/
void GtShell::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtBdrMan::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_MinXSize:
				*ptr=sminxs;
				break;
			case GT_MinYSize:
				*ptr=sminys;
				break;
			case GT_MaxXSize:
				*ptr=smaxxs;
				break;
			case GT_MaxYSize:
				*ptr=smaxys;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtShell::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtShell")==0)
		return(TRUE);
	else
		return(GtBdrMan::IsSubClassOf(cn));
}

/************************************************************************\
*
* Resize shell window.
*
\************************************************************************/
void GtShell::Resize(word xs,word ys)
{
	xsize=xs;
	ysize=ys;
	gfx->win->Resize(xs,ys);
	this->Layout();
}

/************************************************************************\
*
* Rethink layout for shell window.
*
\************************************************************************/
void GtShell::Relayout(void)
{
	GtWidget *obj;
	GtWListBdr *li;
	int xs,ys,mxs,mys,mxxs=GTS_MAX,mxys=GTS_MAX;
	
	if(flags & GTWF_UNBUILT)
		return;
	mxs=1;
	mys=1;
	if((li=(GtWListBdr *)GtSFirst(&man)))
	{
		obj=li->wl.w;
		obj->GetMinSize(&xs,&ys);
		mxs=GTMAX(GTMAX(xs,li->minxs)+li->bleft+li->bright,1);
		mys=GTMAX(GTMAX(ys,li->minys)+li->btop+li->bbot,1);
		obj->GetMaxSize(&xs,&ys);
		mxxs=GTMIN(xs,li->maxxs)+li->bleft+li->bright;
		mxys=GTMIN(ys,li->maxys)+li->btop+li->bbot;
	}
	this->SetMinMaxSize(GTMAX(mxs,sminxs),GTMAX(mys,sminys),
		GTMAX(mxs,GTMIN(mxxs,smaxxs)),GTMAX(mys,GTMIN(mxys,smaxys)));
}

/************************************************************************\
*
* Set event mask for shell.
*
\************************************************************************/
void GtShell::SetEMask(ulong em)
{
	GtBdrMan::SetEMask(em|StructureNotifyMask|KeyPressMask|KeyReleaseMask);
}

/************************************************************************\
*
* Handle changes to shell window.
*
\************************************************************************/
sbool GtShell::Config(XEvent *xe)
{
	Window dmy;
	int xp,yp,oxs,oys;

	if(xe && (xe->type==ConfigureNotify))
	{
		XTranslateCoordinates(dsp->XDsp(),gtwin->XWin(),
													dsp->XDefaultRootWindow(),0,0,&xp,&yp,&dmy);
		xpos=xp;
		ypos=yp;
		oxs=xsize;
		oys=ysize;
		xsize=xe->xconfigure.width;
		ysize=xe->xconfigure.height;
		if((oxs!=xsize) || (oys!=ysize))
			this->Layout();
		return(TRUE);
	}
	return(FALSE);
}

/************************************************************************\
*
* Layout shell window.
*
\************************************************************************/
void GtShell::Layout(void)
{
	GtWidget *obj;
	GtWListBdr *li;
	
	if(flags & GTWF_UNBUILT)
		return;
	if((li=(GtWListBdr *)GtSFirst(&man)))
	{
		obj=li->wl.w;
		obj->SetPosSize(li->bleft,li->bright, 
			(GTMIN(GTMAX(xsize,minxs),maxxs)-li->bleft-li->bright), 
			(GTMIN(GTMAX(ysize,minys),maxys)-li->btop-li->bbot));
	}
}
